<?php

/*
 * @version 6.2.1
 * @package JotCache
 * @category Joomla 3.8
 * @copyright (C) 2010-2018 Vladimir Kanich
 * @license GNU General Public License version 2 or later
 */
defined('JPATH_BASE') or die;

/**
 * Example of JotcacheCustom file (Take it only as code proposal).
 *
 * This class performs three functions for cached pages (Joomla articles and K2 items during cache read as well cache save):
 *
 * 1. incrementing hit counter with each page request. In Articles and K2 options is necessary to disable hit counting
 * because this task will be performed in JotcacheCustom class.
 *
 * 2. adding information about page hits on category pages behind article/K2 item title
 *
 * 3. formating standard date/time for DateCreated on pages of Articles and K2 as relative date/time (difference between
 * DateCreated and actual time of page (browser) request) with easy readable string e.g. '14 hours before'.
 *
 * USAGE:
 * Prepare/change/adjust JotcacheCustom code according your requirements.
 * Copy prepared file into directory JOOMLA_ROOT/plugins/system/jotcache/ as JotcacheCustom.php (without .dist part)
 * Test your code if it fits your requirements.
 *
 */

class JotcacheCustom {

  private function timeRelative($matches) {
    $format = 'l, j F Y H:i';
    $timezone = 'GMT';
    $zone = new DateTimeZone($timezone);
    $now = new DateTime('now', $zone);
    $input = trim($matches[2]);
    $date = DateTime::createFromFormat($format, $input, $zone);
    if (is_object($date) && $now > $date) {
      $difference = $date->diff($now);
      $part = explode('"', $matches[1]);
      $class = substr($part[1], 0, 3);
      $ago = ' ago';
      $result2 = '';
      switch ($class) {
        case 'ite':
          $result2 = $date->format('j F Y');
          if ($difference->y > 0) {
            $result2 = '';
            $result = $date->format('j F Y');
            $ago = '';
            break;
          }
          if ($difference->m > 0) {
            $result = $difference->format('%m') . $this->pluralize($difference->m, 'month');
            break;
          }
          if ($difference->d > 0) {
            $result = $difference->format('%a') . $this->pluralize($difference->d, 'day');
            break;
          }
          if ($difference->h > 0) {
            $result = $difference->format('%h') . $this->pluralize($difference->h, 'hour');
          } else {
            $result = $difference->format('%i') . $this->pluralize($difference->i, 'minute');
          }

          break;
        case 'cat':
        case 'lat':
        case 'tag':
        case 'use':
          if ($difference->y > 0) {
            $result = $difference->format('%y') . $this->pluralize($difference->y, 'year');
            break;
          }
          if ($difference->m > 0) {
            $result = $difference->format('%m') . $this->pluralize($difference->m, 'month');
            break;
          }
          if ($difference->d > 0) {
            $result = $difference->format('%a') . $this->pluralize($difference->d, 'day');
            break;
          }
          if ($difference->h > 0) {
            $result = $difference->format('%h') . $this->pluralize($difference->h, 'hour');
          } else {
            $result = $difference->format('%i') . $this->pluralize($difference->i, 'minute');
          }
          break;
        default:
          break;
      }

      if ($result2) {
        $replacement = ' title="' . $result2 . '">';
        $matches[1] = str_replace(">", $replacement, $matches[1]);
        return $matches[1] . $result . $ago . $matches[3];
      } else {
        return $matches[1] . $result . $ago . $matches[3];
      }
    } else {
      return $matches[1] . $matches[2] . $matches[3];
    }
  }

  private function pluralize($count, $text) {
    if ($text == 'month' || $text == 'year') {
      return (($count == 1) ? (" $text") : (" ${text}s"));
    } else {
      return $text[0];
    }
  }


  public function modifyDataFromCache(&$data) {
    $data = preg_replace_callback('#([<]span\s+class="\w*DateCreated"[>])([^<]*)([<]/span[>])#',
        array($this, 'timeRelative'), $data);
    $this->incrementHit($data);
  }

  public function modifyDataAfterSave(&$data) {
    $data = preg_replace_callback('#([<]span\s+class="\w*DateCreated"[>])([^<]*)([<]/span[>])#',
        array($this, 'timeRelative'), $data);
  }

  private function incrementHit(&$data) {
    $input = JFactory::getApplication()->input;
    $xcom = $input->get('option', '');
    $xview = $input->get('view', '');
    $xid = $input->get('id', 0);
    $xoffset = $input->get('limitstart', 0, 'uint');
    $xhitcount = $input->get('hitcount', 1, 'int');
    if ($xcom == 'com_content' && $xview == 'article' && $xid > 0 && $xoffset == 0 && $xhitcount) {
      $xcache = JFactory::getCache($xcom, 'view');
      $xcache->clean();
      $xdb = JFactory::getDbo();
      $xdb->setQuery(
          'UPDATE #__content' .
          ' SET hits = hits + 1' .
          ' WHERE id = ' . (int)$xid);
      $xdb->query();
      $xdb->setQuery('SELECT hits FROM #__content WHERE id =' . (int)$xid);
      $hits = $xdb->loadResult();
      $data = preg_replace('#(<dd[^>]*class="hits"[^>]*>\s*\w*:)([^<]*)(<\/dd>)#', '${1}' . $hits . ' ${3}', $data);
    }
    if ($xcom == 'com_k2' && ($xview == 'item' || $xview == 'itemlist') && $xoffset == 0 && $xhitcount) {
      $xid = $input->get('id', 0);
//      $xid = JRequest::getInt('id', 0);
      $xid_array = explode(':', $xid);
      $xid = $xid_array[0];
      $xcache = JFactory::getCache('com_k2_extended', 'view');
      $xcache->clean();
      $xdb = JFactory::getDbo();
      $xdb->setQuery(
          'UPDATE #__k2_items' .
          ' SET hits = hits + 1' .
          ' WHERE id = ' . (int)$xid);
      $xdb->query();
      $xdb->setQuery('SELECT hits FROM #__k2_items WHERE id =' . (int)$xid);
      $hits = $xdb->loadResult();
      if ($xview == 'item') {
        $data = preg_replace('#(<span\s+class="itemHits">[^<]*<b>)(\d*)(<\/b>)#', '${1}' . $hits . ' ${3}', $data);
      } else {
        preg_match_all('#<h3\s+class="catItemTitle">\s*<a\s+href="(\/[^"]*)*#', $data, $matches);
        $ids = array();
        $ids2 = array();
        foreach ($matches[1] as $match) {
          $pos = strrpos($match, '/');
          $last = substr($match, $pos);
          preg_match('#\/(\d*)#', $last, $mat);
          if ($mat[1]) {
            $ids[] = $mat[1];
            $ids2[] = $mat[1];
          } else {
            $ids[] = 0;
          }
        }
        $idsImpl = implode("','", $ids2);
        $xdb->setQuery("SELECT hits,id FROM #__k2_items WHERE id IN ('" . $idsImpl . "')");
        $rows = $xdb->loadObjectList();
        $hits = array();
        foreach ($rows as $row) {
          $hits[$row->id] = $row->hits;
        }
        foreach ($ids as $id) {
          if ($id > 0) {
            $data = preg_replace('#(<span\s+class="catItemHits">[^<]*<b>)(\d*)(<\/b>)#', '${1}' . $hits[$id] . ' ${3}', $data, 1);
          } else {
            $data = preg_replace('#(<span\s+class="catItemHits">[^<]*<b>)(\d*)(<\/b>)#', '${1} ${2} ${3}', $data, 1);
          }
        }
      }
    }
  }
}